from leinwand import Leinwand
from abc import ABCMeta,abstractmethod
class Moebel(metaclass=ABCMeta):
    """
    Stellt ein Möbel im Möbelplaner dar.
    """
    def __init__(self,xPosition=20,yPosition=20,breite=40,tiefe=40,winkel=0,randfarbe='schwarz'):
        """
        Erzeugt ein Möbel an der gegebenen Position.
        xPosition: die x-Koordinate der oberen linken Ecke des Möbels
        yPosition: die y-Koordinate der oberen linken Ecke des Möbels
        breite: horizontale Ausdehnung bei winkel 0
        tiefe: vertikale Ausdehnung bei winkel 0
        randfarbe: ist die Randfarbe, mit der das Möbel angezeigt werden soll.
               Dies sollte einer der Farbnamen der Klasse Farben sein.  
        """
        self.__xPosition = xPosition
        self.__yPosition = yPosition
        self.__breite = breite
        self.__tiefe = tiefe
        self.__winkel = winkel
        self.__farbe = 'transparent'
        self.__randfarbe = randfarbe
        self.__formen = []
        self.__leinwand = Leinwand()

    ### alle Get-Methoden
    def gibX(self): return self.__xPosition
    def gibY(self): return self.__yPosition
    def gibB(self): return self.__breite
    def gibT(self): return self.__tiefe
    def gibW(self): return self.__winkel
    def gibFF(self): return self.__farbe
    def gibRF(self): return self.__randfarbe
    def gibFormen(self): return self.__formen
    def gibLW(self): return self.__leinwand
    def istSichtbar(self): return self.__formen != []

    def zeige(self):
        """
        Zeigt dieses Möbel im Möbelplaner an.
        """
        if not self.__formen:
            self.__formen = self.erzeugeFigur()
            xMitte,yMitte=self.__xPosition+self.__breite//2,self.__yPosition+self.__tiefe//2
            for form in self.__formen:
                self.__leinwand.drehe(form,xMitte,yMitte,self.__winkel)
                self.__leinwand.farbeAendern(form,self.__farbe)
                self.__leinwand.randfarbeAendern(form,self.__randfarbe)
    
    @abstractmethod
    def erzeugeFigur(self):
        """
        Hier müssen von Moebel erbende Klassen die Erzeugung der Formen für
        die Darstellung auf der Leinwand vornehmen. Diese Methode soll dann eine
        Liste dieser Formen zur Verwendung durch das Möbel zurückliefern.
        """
        return NotImplemented

    def verberge(self):
        """
        Entfernt dieses Möbel aus der Ansicht im Möbelplaner.
        """
        for form in self.__formen:
            self.__leinwand.loesche(form)
        self.__formen = []

    def drehe(self,umWinkel):
        """
        Dreht das Möbel.
        umWinkel: der Winkel in Grad, um den im Uhrzeigersinn gedreht werden soll
        """
        sichtbar=self.istSichtbar()
        if sichtbar: self.verberge()
        self.__winkel = (self.__winkel + umWinkel) % 360
        if sichtbar: self.zeige()

    def horizontalBewegen(self,entfernung):
        """
        Bewegt den Möbel in horizontaler Richtung auf der Leinwand.
        entfernung: die Anzahl an Pixeln, um die das Möbel nach rechts
                    bewegt werden soll. Für Bewegungen nach links sind
                    negative Zahlen zu verwenden.
        """
        sichtbar=self.istSichtbar()
        if sichtbar: self.verberge()
        self.__xPosition+=entfernung
        if sichtbar: self.zeige()

    def vertikalBewegen(self,entfernung):
        """
        Bewegt das Möbel in vertikaler Richtung auf der Leinwand.
        entfernung: die Anzahl an Pixeln, um die das Möbel nach unten
                    bewegt werden soll. Für Bewegungen nach oben sind
                    negative Zahlen zu verwenden.
        """
        sichtbar=self.istSichtbar()
        if sichtbar: self.verberge()
        self.__yPosition+=entfernung
        if sichtbar: self.zeige()

    def breiteAendern(self,neueBreite):
        """
        Aendert die Breite des Möbels
        neueBreite ersetzt den alten Wert
        """
        sichtbar=self.istSichtbar()
        if sichtbar: self.verberge()
        self.__breite = neueBreite
        if sichtbar: self.zeige()
        
    def tiefeAendern(self,neueTiefe):
        """
        Aendert die Tiefe des Möbels
        neueTiefe ersetzt den alten Wert
        """
        sichtbar=self.istSichtbar()
        if sichtbar: self.verberge()
        self.__tiefe = neueTiefe
        if sichtbar: self.zeige()

    def farbeAendern(self,neueFarbe):
        """
        Aendert die Füllfarbe des Möbels
        """
        sichtbar=self.istSichtbar()
        if sichtbar: self.verberge()
        self.__farbe = neueFarbe
        if sichtbar: self.zeige()

    def randfarbeAendern(self,neueFarbe):
        """
        Aendert die Randfarbe des Möbels
        """
        sichtbar=self.istSichtbar()
        if sichtbar: self.verberge()
        self.__randfarbe = neueFarbe
        if sichtbar: self.zeige()

